#  Licensed Materials - Property of IBM
#  5724-L01
#  Copyright IBM Corporation 2008. All Rights Reserved.
#  US Government Users Restricted Rights - Use, duplication or disclosure
#  restricted by GSA ADP Schedule Contract with IBM Corp.
#

from java.lang import System
from java.io import File
import sys
import re
import sre
import shutil

#------------------------------------------------------------------------------
# a utility method
#------------------------------------------------------------------------------

def wsadminToList(inStr):
  outList=[]
  if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
    inStr = inStr[1:-1]
    tmpList = inStr.split(" ")
  else:
    tmpList = inStr.split("\n")  #splits for Windows or Linux
  for item in tmpList:
    item = item.rstrip();        #removes any Windows "\r"
    if (len(item)>0):
      outList.append(item)
  return outList
#endDef

#------------------------------------------------------------------------------
# check whether the specified application is installed on specified cell
#------------------------------------------------------------------------------
def checkApplicationExist(applicationName):
    exist = 0
    applications = wsadminToList(AdminApp.list())
    for application in applications:
        if (application == applicationName):
            exist = 1
            return exist
    return exist

#------------------------------------------------------------------------------
# get the property value by the the property name; If property not available, return ''
#------------------------------------------------------------------------------
def getPropertyValue(propertyName):
    propertyValue = java.lang.System.getProperty(propertyName)
    if(propertyValue == None):
        return ''
    return propertyValue


def getWASHome():
    return getPropertyValue('was.install.root')

def getProfilePath():
    return getPropertyValue('user.install.root')

def getCellName():
    return getPropertyValue('local.cell')

def getUpdatedNode():
    return getPropertyValue('local.node')

#------------------------------------------------------------------------------
# get the servers defined in specified cell and node; need to filter out the node agent, cluster member
#------------------------------------------------------------------------------
def getServerList(cell,node):
    serverNameList = []
    nodeId = AdminConfig.getid('/Cell:' + cell + '/Node:' + node)
    if(nodeId == ''):
        return serverNameList
    serverList = wsadminToList(AdminConfig.list('Server',nodeId))
    for server in serverList:
        serverName = AdminConfig.showAttribute(server,'name')
        clusterName = AdminConfig.showAttribute(server,'clusterName')
        if(clusterName == None):
            if(serverName != 'nodeagent'):
                serverNameList.append(serverName)
    return serverNameList

#------------------------------------------------------------------------------
# check whether the specified application is installed on specified cell
#------------------------------------------------------------------------------
def checkApplicationExist(applicationName):
    exist = 0
    applications = wsadminToList(AdminApp.list())
    for application in applications:
        if (application == applicationName):
            exist = 1
            return exist
    return exist

#------------------------------------------------------------------------------
# update the application with the specified parameter
#------------------------------------------------------------------------------
def updateApplication(applicationName, earLocation, cell, node, server):
    result = 0
    whetherApplicationExist = checkApplicationExist(applicationName)
    
    print "applicationName="+applicationName+" earLocation="+earLocation+" cell="+cell+" node="+node+" server="+server
    
    if(whetherApplicationExist == 0):
        print applicationName+ " does not exist. Do nothing."
    else:
        print "Updating "+applicationName
        try:
            AdminApp.update(applicationName,"app", ["-operation", "update", "-contents",earLocation])
            AdminConfig.save()  
        except:
            print "Exception occurs in AdminApp.update for "+applicationName    
            result = -1
        
    return result;

#------------------------------------------------------------------------------
# uninstall the application with the specified parameter
#------------------------------------------------------------------------------
def uninstallApplication(applicationName):
    result = 0
    
    print "applicationName="+applicationName
    # Uninstall the app if it already exists.
    for appName in wsadminToList(AdminApp.list( )):
            if (cmp(appName, applicationName) == 0):
                    try:
                        AdminApp.uninstall(appName )
                        AdminConfig.save( )
                        print "Uninstalled "+appName
                    except:
                        print "Exception occurs in AdminApp.uninstall for "+appName    
                        result = -1
            #endIf 
    #endFor 
    
    return result;


#------------------------------------------------------------------------------
# Install the application with the specified parameter
#------------------------------------------------------------------------------
def installApplication(applicationName, earLocation, cell, node, server):
    result = 0
    
    print "applicationName="+applicationName+" earLocation="+earLocation+" cell="+cell+" node="+node+" server="+server
    # Uninstall the app first if it already exists.
    result = uninstallApplication(applicationName)
    if(result == 0):
        try:
            attr = ["-server", server]
            AdminApp.install(earLocation, attr )
            AdminConfig.save( )
        except:
            print "Exception occurs in AdminApp.install for earLocation="+earLocation+" attr="+str(attr)    
            result = -1

    return result;

#------------------------------------------------------------------------------
# Install the application or update the applicaiton if it's already installed
#------------------------------------------------------------------------------
def installOrUpdateApplication(applicationName, earLocation, cell, node, server):
    result = 0
    whetherApplicationExist = checkApplicationExist(applicationName)
    
    print "applicationName="+applicationName+" earLocation="+earLocation+" cell="+cell+" node="+node+" server="+server
    
    if(whetherApplicationExist == 0):
        print applicationName+ " does not exist. Install it."
        result = installApplication(applicationName, earLocation, cell, node, server) 
    else:
        print applicationName+ " exists. Update it."
        result = updateApplication(applicationName, earLocation, cell, node, server)
        
    return result

#------------------------------------------------------------------------------
# Copy registration files
#------------------------------------------------------------------------------
def copyRegistrationFile(srcPath, destPath, file):
    result = 0
    try:
        destDir = File(destPath)
        if not destDir.exists():
            endpointDir.mkdirs()
        print "copy "+file+" from " + srcPath + " to " + destPath
        shutil.copy( srcPath + file, destPath + file);

    except:
        print "warning: Failed to copy "+file+" end point registration file."
        result = -1
    return result

#------------------------------------------------------------------------------
# Copy widget registration files
#------------------------------------------------------------------------------
def copyServiceEndpointFiles():
    result = 0
    try:
    	srcPath  = getWASHome()     + "/BusinessSpace/wpbs/registryData/";
    	destPath = getProfilePath() + "/BusinessSpace/registryData/";
 
# The Widget files are not updated since they are contained in the Widget ear also. The ones in the private
# location are for custom changes. If the user made custom changes, then we would not know and if we did do
# a copy, then we would overwrite the users custom changes. Therefore, copying over of the widget files will
# have to be a manual step and only if the user made custom changes.    	
#        copyRegistrationFile( srcPath, destPath, "fabricWidgets.xml");
#        copyRegistrationFile( srcPath, destPath, "googleWidgets.xml");
#        copyRegistrationFile( srcPath, destPath, "monitorWidgets.xml");
#        copyRegistrationFile( srcPath, destPath, "productivityWidgets.xml");
        copyRegistrationFile( srcPath, destPath, "pubserverEndpoints.xml");
#        copyRegistrationFile( srcPath, destPath, "pubserverWidgets.xml");
#        copyRegistrationFile( srcPath, destPath, "wpsWidgets.xml");

    except:
        print "warning: Failed to copy service end point registration files."
        result = -1
    return result

#------------------------------------------------------------------------------
# Install Business Space
#------------------------------------------------------------------------------
def installBSpace(cellName,nodeName,serverName):
    result = 0
    try:
        if(checkApplicationExist("BusinessSpaceManager") ==  1 and checkApplicationExist("IBM_BSPACE_WIDGETS") == 1):
            print "BusinessSpaceManager and IBM_BSPACE_WIDGETS already deployed. Do nothing."
        else:
            optList = "-nodeName "+nodeName+" -serverName "+serverName
            print "installBusinessSpace optList="+optList
            AdminTask.installBusinessSpace(optList)
            # BUGBUG
            # Sometimes, we see this exception.
            # WASX7015E: Exception running command: "AdminConfig.save()"; exception information:
            # java.util.ConcurrentModificationException: java.util.ConcurrentModificationException
            # Let's delay 10 seconds before save.
            sleep(10)
            AdminConfig.save()
    except:
        print "warning: Install BSpace failed." 
        result = -1
        
    return result

#------------------------------------------------------------------------------
# Install Business Space (the WAS way) to workaround 530928
#------------------------------------------------------------------------------
def installBSpace2(earLocationPrefix,cellName,nodeName,serverName):
    result = 0
    rc1 = 0
    rc2 = 0
    try:
        if(checkApplicationExist("BusinessSpaceManager") ==  1):
            print "BusinessSpaceManager already deployed. Do nothing."
        else:
            print "Installing BusinessSpaceManager"
            rc1 = installApplication("BusinessSpaceManager", earLocationPrefix+"BSpaceManager.ear", cell, node, server)
            print "Done installing BusinessSpaceManager"
    except:
        print "warning: Install BusinessSpaceManager failed." 
        rc1 = -1

    try:
        if(checkApplicationExist("IBM_BSPACE_WIDGETS") == 1):
            print "IBM_BSPACE_WIDGETS already deployed. Do nothing."
        else:
            print "Installing IBM_BSPACE_WIDGETS"
            rc2 = installApplication("IBM_BSPACE_WIDGETS", earLocationPrefix+"BSpaceWidgets.ear", cell, node, server)
            print "Done installing IBM_BSPACE_WIDGETS"
    except:
        print "warning: Install IBM_BSPACE_WIDGETS failed." 
        rc2 = -1
        
    if(rc1 == -1 or rc2 == -1):
        result = -1
        
    return result

#------------------------------------------------------------------------------
# Configure Business Space
#------------------------------------------------------------------------------
def configureBSpace(cellName,nodeName,serverName):
    result = 0

    try:
        found = "false"
        datasources = AdminTask.listDatasources("-scope Server")
        if(datasources != ""): 
            datasourceIds = datasources.splitlines()
            for i in range(len(datasourceIds)):
                print "datasourceIds="+str(datasourceIds[i])
                jndiName = AdminConfig.showAttribute(datasourceIds[i],"jndiName")
                print "jndiName ="+jndiName
                if(jndiName == "jdbc/bpm/BusinessSpace"):
                    found = "true"
                    break
                    
        print "jdbc/bpm/BusinessSpace found? "+found
        
        if(found == "true"):
            print "BusinessSpace already configured. Do nothing."
        else:
            optList = "-nodeName "+nodeName+" -serverName "+serverName+" -productTypeForDatasource WPBS"
            print "configureBusinessSpace optList="+optList
            AdminTask.configureBusinessSpace(optList)
            AdminConfig.save()
    except:
        print "warning: Configure BSpace failed." 
        result = -1
        
    return result

#
# --- Main
#
washome =  getWASHome()
cell = getCellName()
node = getUpdatedNode()
serverList = getServerList(cell,node)

commonLocationPrefix = washome+"/installableApps/";
appLocationPrefix = washome+"/installableApps.wpbs/";

rc = 0

rc += copyServiceEndpointFiles()

for server in serverList:
#    rc += uninstallApplication("IBM_BSPACE_WIDGETS")
    
    #Install/Configure BSpace
    #rc += installBSpace(cell,node,server)
    #Workaround on BSpace defect 530928
    
# We do not want to install. Only update if it already exists.    
#    rc += installBSpace2(commonLocationPrefix,cell,node,server)

# Update Business Space Ears if they are found.
	rc += updateApplication("BusinessSpaceManager", commonLocationPrefix+"BSpaceManager.ear", cell, node, server)
	rc += updateApplication("IBM_BSPACE_WIDGETS", commonLocationPrefix+"BSpaceWidgets.ear", cell, node, server)
	
# Since we are updating the applications, we do now want to touch the configuration (the user may
# have customized that also.	
#    rc += configureBSpace(cell,node,server)

	rc += updateApplication("IBM_WPBS_PUBSERVER", appLocationPrefix+"PublishingServer.ear", cell, node, server)

if(rc != 0):
    raise ValueError("Updating application failed.")
